//Standard libraries
#include <iostream.h>
#include <string>

//ObjectModelerBase Framework
#include "CATSession.h"
#include "CATSessionServices.h"
#include "CATDocument.h"
#include "CATDocumentServices.h"
#include "CATInit.h"

//MecModInterfaces Framework
#include "CATIPrtContainer.h"
#include "CATIPrtPart.h"

//SketcherInterfacesFramework
#include "CATISketchFactory.h"
#include "CATISketch.h"
#include "CATI2DWFFactory.h"
#include "CATI2DCurve.h"
#include "CATI2DPoint.h"

//ObjectSpecsModeler
#include "CATISpecObject.h"			

//Mathematics Framework
#include "CATMathPlane.h"

//PartInterfaces Framework
#include "CATIPrtFactory.h"
#include "CATIPad.h"
#include "CATLimitDefs.h"

// LiteralFeatures FW
#include "CATIParmPublisher.h"
#include "CATICkeParmFactory.h"
#include "CATICkeParm.h"

// KnowledgeInterfaces FW
#include "CATIVisitor.h"

void main()
{
// Variables
	double dLenX, dLenY, dLenZ;
	char cName[256];

// Gets the user name
	cout << "Enter your name: ";
	cin >> cName;

// Defines cube size:
	dLenX = 25.4;
	dLenY = 25.4;
	dLenZ = 25.4;
	
// Opening a session
	CATSession * pSampleSession = NULL;
	Create_Session("SampleSession",pSampleSession);

// Opens a CATPart document
	CATDocument * pDoc = NULL;
	CATDocumentServices::New("CATPart",pDoc);

// Initializes the data
	CATInit_var spInitOnDoc(pDoc);

// Retrieves the root container
	CATIPrtContainer * piPrtCont = 
		(CATIPrtContainer*) spInitOnDoc->GetRootContainer("CATIPrtContainer");

// Retrieves the sketch factory
	CATISketchFactory_var spSketchFactOnPrtCont(piPrtCont);

// Creates the sketch plane and retrieves the part reference planes
	CATIPrtPart_var spPart = piPrtCont->GetPart();
	CATListValCATISpecObject_var spListRefPlanes = spPart -> GetReferencePlanes();

// Defines the xy plane as the first sketch plane
	CATISpecObject_var spSketchPlane = spListRefPlanes[1];

// Instantiate the sketch
	CATISketch_var spSketch = spSketchFactOnPrtCont->CreateSketch(spSketchPlane);

// Retrieves the 2D Factory
	CATI2DWFFactory_var spWF2DFactOnSketch(spSketch);

// Creates the points
    double pt1[2] = {dLenX, dLenY},
	       pt2[2] = {0., dLenY},
	       pt3[2] = {0., 0.},
	       pt4[2] = {dLenX, 0.};	

// Opens the sketch and draws lines
	spSketch->OpenEdition();

	CATISpecObject_var spLine1 = spWF2DFactOnSketch->CreateLine(pt1,pt2);
	CATISpecObject_var spLine2 = spWF2DFactOnSketch->CreateLine(pt2,pt3);
	CATISpecObject_var spLine3 = spWF2DFactOnSketch->CreateLine(pt3,pt4);
	CATISpecObject_var spLine4 = spWF2DFactOnSketch->CreateLine(pt4,pt1);

//Defines end points
	CATI2DCurve_var spCurveOnLine1(spLine1);
	spCurveOnLine1->GetStartPoint();
	spCurveOnLine1->GetEndPoint();
	CATI2DCurve_var spCurveOnLine2(spLine2);
	spCurveOnLine2->GetStartPoint();
	spCurveOnLine2->GetEndPoint();
	CATI2DCurve_var spCurveOnLine3(spLine3);
	spCurveOnLine3->GetStartPoint();
	spCurveOnLine3->GetEndPoint();
	CATI2DCurve_var spCurveOnLine4(spLine4);
	spCurveOnLine4->GetStartPoint();
	spCurveOnLine4->GetEndPoint();

//Closes the sketch session
	spSketch->CloseEdition();

//Defines the direction of the pad
    CATMathDirection dirZ(0., 0., 1.);

//Retrieves the Mechanical Design Factory to create pad
    CATIPrtFactory_var spPrtFactOnPrtCont(piPrtCont);


//Creates pad
    CATISpecObject_var spSpecObj = spPrtFactOnPrtCont->CreatePad(spSketch);

	CATIPad_var spPadOnSpecObj(spSpecObj);
	spPadOnSpecObj->ModifyDirection(dirZ);
    spPadOnSpecObj->ModifyEndType(catOffsetLimit);
	spPadOnSpecObj->ModifyEndOffset(dLenZ);
	spPadOnSpecObj->ModifyStartType(catOffsetLimit);
	spPadOnSpecObj->ModifyStartOffset(0.);

//Builds pad "Update"
    spSpecObj->Update();

	CATIParmPublisher_var spPublisher = spPart;
	
	CATICkeParmFactory_var spParamFact = piPrtCont;
	piPrtCont->Release();
    piPrtCont = NULL;
	
    CATLISTV(CATBaseUnknown_var) ParamList;
    CATIVisitor_var spStandardVisitor = 
		spParamFact->CreateStandardVisitor(IID_CATICkeParm, &ParamList);
	
	spPublisher->VisitChildren(spStandardVisitor, 1);

	CATICkeParm_var tempParam = NULL_var;
	CATUnicodeString tempUnicode = "CATPart1\\Definition";

	for (int j=1; j<=ParamList.Size(); j++)
	{
		tempParam = ParamList[j];
		if (tempUnicode == tempParam->Name())
		{
			tempParam->Valuate(cName);
		}
	}

	spSpecObj->Update();

//Saves the pad
	char tempName[256];
	sprintf(tempName,"C:\\%s.CATPart",cName);
    CATDocumentServices::SaveAs(*pDoc, tempName);

//Closes the section
    ::Delete_Session("SampleSession");

	cout << "Program ended successfully" << endl;
	cout << "Part was saved at " << tempName << endl;
}